@@ -1,43 +1,47 @@
-/*****************************************************************************
- *
- * Win32 version of the MMX optimized routines. All MMX optimized functions
- * have been gathered into this single source code file, regardless to their 
- * class or original source code file, in order to ease porting the library
- * to other compiler and processor platforms.
- *
- * This file is to be compiled in Windows platform with Microsoft Visual C++ 
- * Compiler. Please see 'mmx_gcc.cpp' for the gcc compiler version for all
- * GNU platforms.
- *
- * Author        : Copyright (c) Olli Parviainen
- * Author e-mail : oparviai @ iki.fi
- * File created  : 13-Jan-2002
- *
- * Last changed  : $Date: 2003/11/17 17:56:23 $
- * File revision : $Revision: 1.11 $
- *
- * $Id: mmx_win.cpp,v 1.11 2003/11/17 17:56:23 Olli Exp $
- *
- * License :
- * 
- *  SoundTouch sound processing library
- *  Copyright (c) Olli Parviainen
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- *****************************************************************************/
+////////////////////////////////////////////////////////////////////////////////
+///
+/// Win32 version of the MMX optimized routines. All MMX optimized functions
+/// have been gathered into this single source code file, regardless to their 
+/// class or original source code file, in order to ease porting the library
+/// to other compiler and processor platforms.
+///
+/// This file is to be compiled in Windows platform with Microsoft Visual C++ 
+/// Compiler. Please see 'mmx_gcc.cpp' for the gcc compiler version for all
+/// GNU platforms.
+///
+/// Author        : Copyright (c) Olli Parviainen
+/// Author e-mail : oparviai @ iki.fi
+/// SoundTouch WWW: http://www.iki.fi/oparviai/soundtouch
+///
+////////////////////////////////////////////////////////////////////////////////
+//
+// Last changed  : $Date: 2005/02/06 11:45:31 $
+// File revision : $Revision: 1.15 $
+//
+// $Id: mmx_win.cpp,v 1.15 2005/02/06 11:45:31 Olli Exp $
+//
+////////////////////////////////////////////////////////////////////////////////
+//
+// License :
+//
+//  SoundTouch audio processing library
+//  Copyright (c) Olli Parviainen
+//
+//  This library is free software; you can redistribute it and/or
+//  modify it under the terms of the GNU Lesser General Public
+//  License as published by the Free Software Foundation; either
+//  version 2.1 of the License, or (at your option) any later version.
+//
+//  This library is distributed in the hope that it will be useful,
+//  but WITHOUT ANY WARRANTY; without even the implied warranty of
+//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+//  Lesser General Public License for more details.
+//
+//  You should have received a copy of the GNU Lesser General Public
+//  License along with this library; if not, write to the Free Software
+//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//
+////////////////////////////////////////////////////////////////////////////////
 
 #include "STTypes.h"
 
@@ -45,6 +49,7 @@
 #error "wrong platform - this source code file is exclusively for Win32 platform"
 #endif
 
+using namespace soundtouch;
 
 #ifdef ALLOW_MMX
 // MMX routines available only with integer sample type    
@@ -62,9 +67,9 @@
 extern int scanOffsets[4][24];
 
 // Calculates cross correlation of two buffers
-_inline int TDStretchMMX::calculateCrossCorrelation(const short *pV1, const short *pV2) const
+long TDStretchMMX::calcCrossCorrStereo(const short *pV1, const short *pV2) const
 {
-    int corr;
+    long corr;
     uint local_overlapLength = overlapLength;
     uint local_overlapDividerBits = overlapDividerBits;
 
@@ -186,7 +191,7 @@ _inline int TDStretchMMX::calculateCrossCorrelation(const short *pV1, const shor
 
 
 
-_inline void _clearEMMS()
+void TDStretchMMX::clearCrossCorrState()
 {
     _asm EMMS;
 }
@@ -194,96 +199,6 @@ _inline void _clearEMMS()
 
 
 
-// MMX-optimized version of the function "seekBestOverlapPositionStereo"
-uint TDStretchMMX::seekBestOverlapPositionStereo(const short *other)
-{
-    uint bestoffs;
-    int bestcorr, corr;
-    uint corrOffset, i;
-
-    // Slopes the amplitude of the "midBuffer" samples
-    slopeReferenceSamplesStereo();
-
-    bestcorr = INT_MIN;
-    bestoffs = 0;
-    corrOffset = 0;
-
-    // Scans for the best correlation value by testing each possible position
-    // over the permitted range.
-    for (i = 0; i < seekLength; i ++) 
-    {
-            // Calculates the cross-correlation value for the mixing position 
-            // corresponding to "i"
-            corr = calculateCrossCorrelation(other + 2 * i, pRefMidBuffer);
-
-            // Checks for the highest correlation value
-            if (corr > bestcorr) 
-            {
-                bestoffs = i;
-                bestcorr = corr;
-            }
-        }
-    // Clear the MMX state before leaving the subroutine
-    _clearEMMS();
-    
-    return bestoffs;
-}
-
-
-
-
-// MMX-optimized version of the function "seekBestOverlapPositionStereoQuick"
-uint TDStretchMMX::seekBestOverlapPositionStereoQuick(const short *other)
-{
-    short *local_refMidBuffer = pRefMidBuffer;
-    uint bestpos, scancount, i;
-    int bestcorr,corr;
-    uint corrPos, tempPos;
-
-    // Slopes the amplitude of the "midBuffer" samples
-    slopeReferenceSamplesStereo();
-
-    bestcorr = INT_MIN;
-    bestpos = scanOffsets[0][0];
-    corrPos = 0;
-    tempPos = 0;
-
-    // Scans for the best correlation value using four-pass hierarchical search.
-    //
-    // The look-up table "scans" has hierarchical position adjusting steps.
-    // In first pass the routine searhes for the highest correlation with 
-    // relatively coarse steps, then rescans the neighbourhood of the highest
-    // correlation with better resolution and so on.
-
-    for (scancount = 0;scancount < 4; scancount ++) 
-    {
-        i = 0;
-        while (scanOffsets[scancount][i]) 
-        {
-            tempPos = corrPos + scanOffsets[scancount][i];
-            if (tempPos >= seekLength) break;
-
-            // Calculates correlation value for the mixing position corresponding
-            // to "tempPos"
-            corr = calculateCrossCorrelation(other + 2 * tempPos, pRefMidBuffer);
-            // Checks for the highest correlation value
-            if (corr > bestcorr) 
-            {
-                bestpos = tempPos;
-                bestcorr = corr;
-            }
-
-            i ++;
-        }
-        corrPos = bestpos;
-    }
-    // Clear the MMX state before leaving the subroutine
-    _clearEMMS();
-    
-    return bestpos;
-}
-
-
 
 // MMX-optimized version of the function overlapStereo
 void TDStretchMMX::overlapStereo(short *output, const short *input) const
@@ -427,6 +342,7 @@ void FIRFilterMMX::setCoefficients(const short *coeffs, uint newLength, uint uRe
     FIRFilter::setCoefficients(coeffs, newLength, uResultDivFactor);
 
     // Ensure that filter coeffs array is aligned to 16-byte boundary
+    delete[] filterCoeffsUnalign;
     filterCoeffsUnalign = new short[2 * newLength + 8];
     filterCoeffsAlign = (short *)(((uint)filterCoeffsUnalign + 15) & -16);
 
@@ -456,6 +372,8 @@ uint FIRFilterMMX::evaluateFilterStereo(short *dest, const short *src, const uin
     uint local_resultDivider = resultDivFactor;
     short *local_filterCoeffs = (short*)filterCoeffsAlign;
 
+    if (local_length < 2) return 0;
+
     _asm 
     {
         ; Load (num_samples-aa_filter_length)/2 to edi as a i
